#include "dialog.h"
#include "ui_dialog.h"
#include <VDream/Common/VDebugNew.h>

Dialog::Dialog(QWidget *parent) :
  QDialog(parent),
  ui(new Ui::Dialog)
{
  ui->setupUi(this);

  lastProgress = 0;
  loadingCount = 0;
  wal = new WebAutoLogin(this);

  this->loadFromDefaultDoc("main/coord");
}

Dialog::~Dialog()
{
  this->saveToDefaultDoc("main/coord");
  SAFE_DELETE(wal);
  delete ui;
}

void Dialog::load(VXML xml)
{
  QRect rect = this->geometry();
  rect.setLeft(xml.getInt("left",     rect.left()));
  rect.setTop(xml.getInt("top",       rect.top()));
  rect.setWidth(xml.getInt("width",   rect.width()));
  rect.setHeight(xml.getInt("height", rect.height()));
  this->setGeometry(rect);

  ui->leGo->setText(xml.getStr("url", ui->leGo->text()));
}

void Dialog::save(VXML xml)
{
  QRect rect = this->geometry();
  xml.setInt("left",   rect.left());
  xml.setInt("top",    rect.top());
  xml.setInt("width",  rect.width());
  xml.setInt("height", rect.height());

  xml.setStr("url", ui->leGo->text());
}

void Dialog::on_btnGo_clicked()
{
  LOG_DEBUG("");
  lastProgress = 0;
  ui->webView->load(QUrl(ui->leGo->text()));
}

void Dialog::on_webView_iconChanged()
{
  LOG_DEBUG("");
}

void Dialog::on_webView_linkClicked(const QUrl &arg1)
{
  LOG_DEBUG("%s", qPrintable(arg1.toString()));
}

void Dialog::on_webView_loadFinished(bool arg1)
{
  LOG_DEBUG("%d", (int)arg1);
  if (lastProgress != 100) return;
  loadingCount--;
  wal->webViewEvent.setEvent();
}

void Dialog::on_webView_loadProgress(int progress)
{
  lastProgress = progress;
  ui->progressBar->setValue(progress);
  if (progress != 100) return;
  LOG_DEBUG("%d", progress);
}

void Dialog::on_webView_loadStarted()
{
  LOG_DEBUG("");
  ui->progressBar->setValue(0);
  loadingCount++;
}

void Dialog::on_webView_selectionChanged()
{
  LOG_DEBUG("");
}

void Dialog::on_webView_statusBarMessage(const QString &text)
{
  if (text == "") return;
  LOG_DEBUG("%s", qPrintable(text));
}

void Dialog::on_webView_titleChanged(const QString &title)
{
  this->setWindowTitle(title);
}

void Dialog::on_webView_urlChanged(const QUrl &arg1)
{
  QString url = arg1.toString();
  LOG_DEBUG("%s", qPrintable(url));
  ui->leGo->setText(url);
}

void Dialog::on_webView_customContextMenuRequested(const QPoint &pos)
{
  LOG_DEBUG("");
}
