#include "webautologin.h"
#include "dialog.h"
#include "ui_dialog.h"

// ----------------------------------------------------------------------------
// WebAutoLogin
// ----------------------------------------------------------------------------
WebAutoLogin::WebAutoLogin(Dialog* dialog)
{
  qRegisterMetaType<QWebElement>("QWebElement");
  qRegisterMetaType<QWebElement*>("QWebElement*");

  QObject::connect(this, SIGNAL(_go(QString)), this, SLOT(go(QString)));
  QObject::connect(this, SIGNAL(_clickElement(QWebElement*)), this, SLOT(clickElement(QWebElement*)));
  QObject::connect(this, SIGNAL(_setAttribute(QWebElement*,QString)), this, SLOT(setAttribute(QWebElement*,QString)));
  QObject::connect(this, SIGNAL(_pressEnterElement(QWebElement*)), this, SLOT(pressEnterElement(QWebElement*)));
  QObject::connect(this, SIGNAL(_applicationTerminate()), this, SLOT(applicationTerminate()));

  m_dialog = dialog;

  url     = "";
  id      = "";
  pw      = "";

  VApp& app = VApp::instance();
  if (app.argc() > 1) { url = app.argv()[1]; dialog->ui->leGo->setText(url); }
  if (app.argc() > 2) id  = app.argv()[2];
  if (app.argc() > 3) pw  = app.argv()[3];
  waitInterval = 1000;
  pageInterval = 30000;

  acceptIDTagNames.append("input");
  acceptIDTagNames.append("button");

  acceptIDAttributeNames.append("id");
  acceptIDAttributeNames.append("name");

  acceptIDAttributeValues.append("id");
  acceptIDAttributeValues.append("usrid");
  acceptIDAttributeValues.append("userid");
  acceptIDAttributeValues.append("user_id");
  acceptIDAttributeValues.append("uid");
  acceptIDAttributeValues.append("loginname");
  acceptIDAttributeValues.append("email");
  acceptIDAttributeValues.append("login_email");
  acceptIDAttributeValues.append("dvid");      // for devpia
  acceptIDAttributeValues.append("edit-name"); // for kldp
  acceptIDAttributeValues.append("txtuserid"); // for csc.secui.com
  acceptIDAttributeValues.append("signin-email"); // for twitter.com

  this->loadFromDefaultDoc("WebAutoLogin");

  runThread().open();
}

WebAutoLogin::~WebAutoLogin()
{
  runThread().close();
  this->saveToDefaultDoc("WebAutoLogin");
}

void WebAutoLogin::load(VXML xml)
{
  waitInterval = xml.getInt("waitInterval", waitInterval);
  pageInterval = xml.getInt("pageInterval", pageInterval);
  acceptIDTagNames        = xml.getStr("acceptIDTagNames",        acceptIDTagNames.join(",")).split((","));
  acceptIDAttributeNames  = xml.getStr("acceptIDAttributeNames",  acceptIDAttributeNames.join(",")).split((","));
  acceptIDAttributeValues = xml.getStr("acceptIDAttributeValues", acceptIDAttributeValues.join(",")).split((","));
}

void WebAutoLogin::save(VXML xml)
{
  xml.setInt("waitInterval", waitInterval);
  xml.setInt("pageInterval", pageInterval);
  xml.setStr("acceptIDTagNames",        acceptIDTagNames.join(","));
  xml.setStr("acceptIDAttributeNames",  acceptIDAttributeNames.join(","));
  xml.setStr("acceptIDAttributeValues", acceptIDAttributeValues.join(","));
}

bool WebAutoLogin::waitSleep()
{
  waitEvent.resetEvent();
  return !waitEvent.wait(waitInterval);
}

void WebAutoLogin::run()
{
  LOG_DEBUG("thread beg");

  bool res;
  QWebElement idElement;
  QWebElement pwElement;


  //
  // go to webpage
  //
  if (!waitSleep()) goto _end;
  res = go(url);
  if (!res)
  {
    LOG_ERROR("go to webpage(%s) failed", qPrintable(url));
    goto _end;
  }
  LOG_DEBUG("go to webpage(%s) succeed", qPrintable(url));

  //
  // find id and pw element
  //
  if (!waitSleep()) goto _end;
  if (!findIDAndPWElement(idElement, pwElement))
  {
    QWebElement loginElement = findLoginElement();
    if (loginElement.isNull())
    {
      LOG_ERROR("can not find login element");
      goto _end;
    }
  }
  LOG_DEBUG("find id and pw element succeed");

  //
  // press enter pw element
  //
  if (!waitSleep()) goto _end;
  res = emit _pressEnterElement(&pwElement);
  if (!res)
  {
    LOG_ERROR("can not press enter pw element");
    goto _end;
  }
  LOG_DEBUG("press enter pw element succeed");

  //
  // login
  //
  if (!webViewEvent.wait(pageInterval))
  {
    LOG_ERROR("can not login page");
    goto _end;
  }
  LOG_DEBUG("login succeed");

_end:
  waitSleep();
  LOG_DEBUG("thread end");
  emit _applicationTerminate();
}

bool WebAutoLogin::go(QString url)
{
  LOG_DEBUG(""); // gilgil temp
  m_dialog->ui->leGo->setText(url);
  webViewEvent.resetEvent();
  m_dialog->ui->btnGo->click();
  bool res = true;
  while (true)
  {
    if (!webViewEvent.wait(pageInterval)) return false;
    waitSleep();
    if (m_dialog->loadingCount == 0) break;
    LOG_DEBUG("loading count=%d", m_dialog->loadingCount);
  }
  return res;
}

bool WebAutoLogin::findIDAndPWElement(QWebElement& idElement, QWebElement& pwElement)
{
  QWebElement rootElement = m_dialog->ui->webView->page()->mainFrame()->documentElement();

  pwElement = findPWElement(rootElement);
  if (pwElement.isNull()) return false;
  LOG_DEBUG("pw element found: %s", qPrintable(pwElement.toOuterXml()));

  bool idFound = false;
  int  depth = 0;
  QWebElement parent = pwElement.parent();
  while (!parent.isNull())
  {
    depth++;
    if (depth > 16) break;
    idElement = findIDElement(parent, depth);
    if (!idElement.isNull())
    {
      LOG_DEBUG("id element found: %s", qPrintable(idElement.toOuterXml()));
      idFound = true;
      break;
    }
    parent = parent.parent();
  }
  if (!idFound) return false;

  if (!pwElement.geometry().isValid())
  {
    QRect rect = pwElement.geometry();
    LOG_ERROR("pwElement geometry is invalid %d %d %d %d", rect.left(), rect.top(), rect.width(), rect.height());
    return false;
  }
  if (!idElement.geometry().isValid())
  {
    QRect rect = idElement.geometry();
    LOG_ERROR("idElement geometry is invalid %d %d %d %d", rect.left(), rect.top(), rect.width(), rect.height());
    return false;
  }

  emit _clickElement(&idElement);
  waitSleep();
  emit _setAttribute(&idElement, id);
  waitSleep();
  emit _clickElement(&pwElement);
  waitSleep();
  emit _setAttribute(&pwElement, pw);
  waitSleep();

  return true;
}

QWebElement WebAutoLogin::findPWElement(QWebElement element)
{
  QString value = element.attribute("type");
  if (value == "password") return element;

  QWebElement childElement = element.firstChild();
  while (!childElement.isNull())
  {
    QWebElement res = findPWElement(childElement);
    if (!res.isNull()) return res;
    childElement = childElement.nextSibling();
  }
  return QWebElement();
}

QWebElement WebAutoLogin::findIDElement(QWebElement element, int depth)
{
  bool tagFound = false;
  QString elementTagName = element.tagName().toLower();
  foreach (QString tagName, acceptIDTagNames)
  {
    if (tagName == elementTagName)
    {
      tagFound = true;
      break;
    }
  }

  if (tagFound)
  {
    foreach (QString acceptIDAttributeName, this->acceptIDAttributeNames)
    {
      QString attribute = element.attribute(acceptIDAttributeName);
      if (attribute == "") continue;
      attribute = attribute.toLower();
      foreach (QString acceptIDAttributeValue, this->acceptIDAttributeValues)
      {
        if (attribute == acceptIDAttributeValue)
        {
          // LOG_DEBUG("id element found %s", qPrintable(element.toOuterXml())); // gilgil temp
          return element;
        }
      }
    }
  }

  if (depth <= 0) return QWebElement();

  QWebElement childElement = element.firstChild();
  while (!childElement.isNull())
  {
    QWebElement res = findIDElement(childElement, depth - 1);
    if (!res.isNull()) return res;
    childElement = childElement.nextSibling();
  }
  return QWebElement();
}

QWebElement WebAutoLogin::findLoginElement()
{
  return QWebElement(); // gilgil temp
}

bool WebAutoLogin::clickElement(QWebElement* element)
{
  LOG_DEBUG("%p", element);
  bool res;
  QRect rect = element->geometry();
  QMouseEvent press(
    QMouseEvent::MouseButtonPress, rect.center(),
    Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
  res = QApplication::sendEvent(m_dialog->ui->webView, &press);
  if (!res)
  {
    LOG_ERROR("sendEvent(press) return false;");
    return false;
  }

  QMouseEvent release(
    QMouseEvent::MouseButtonRelease, rect.center(),
    Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
  res = QApplication::sendEvent(m_dialog->ui->webView, &release);
  if (!res)
  {
    LOG_ERROR("sendEvent(release) return false;");
    return false;
  }
  return true;
}

bool WebAutoLogin::setAttribute(QWebElement* element, QString value)
{
  LOG_DEBUG("%s %s", qPrintable(element->tagName()), qPrintable(value));
  element->setAttribute("value", value);
  return true;
}

bool WebAutoLogin::pressEnterElement(QWebElement* element)
{
  LOG_DEBUG("");
  QKeyEvent press(QKeyEvent::KeyPress, Qt::Key_Return, Qt::NoModifier, "Return", false, 0 );
  bool res = QApplication::sendEvent(m_dialog->ui->webView, &press);
  if (!res)
  {
    LOG_ERROR("sendEvent(press) return false;");
    return false;
  }
  QKeyEvent release(QKeyEvent::KeyRelease, Qt::Key_Return, Qt::NoModifier, "Return", false, 0 );
  res = QApplication::sendEvent(m_dialog->ui->webView, &release);
  if (!res)
  {
    LOG_ERROR("sendEvent(release) return false;");
    return false;
  }
  return true;
}

bool WebAutoLogin::applicationTerminate()
{
  LOG_DEBUG("");
  m_dialog->close();
  return true;
}
